class ExamsController < ApplicationController

  before_filter :find_exam, :only => [:show, :edit, :update, :destroy, :question_new, :question_create, :end]
  before_filter :find_answer, :only => [:question_new, :question_create]

  def allows_to?(action)
    allows_to = case(action)
    when "index", "show", "new", "create" then current_user.has_role?("examiner")
    when "edit", "update", "destroy" then current_user.has_role?("admin")
    when "question_new", "question_create", "end" then current_user.has_role?("candidate")
    else false
    end
    return(allows_to)
  end

  def index
    @exams = Exam.paginate(:order => "process ASC, created_at ASC", :page => params[:page], :per_page => Setting.elements_per_page)
  end

  def show
  end

  def new
    @categories = Category.all
    @categories.delete_if{|category| category.active_questions.empty?}
    @exam = Exam.new
    @exam.factor_ok = Setting.exams_factor_ok
    @exam.factor_ko = Setting.exams_factor_ko
    @exam.factor_empty = Setting.exams_factor_empty
    respond_to do |format|
      format.html
      format.js
    end
  end

  def create
    mix_questions = (defined?(params[:mix_questions]) and (params[:mix_questions] == "1"))
    mix_choices = (defined?(params[:mix_choices]) and (params[:mix_choices] == "1"))
    just_print = (defined?(params[:just_print]) and (params[:just_print] == "1"))
    @categories = {}
    params[:categories].each do |info|
      category_id = info[0].to_i
      count = info[1].to_i
      category = Category.find(category_id)
      max_count = category.questions.length
      count = 0 if count < 0
      count = max_count if count > max_count
      questions = category.active_questions
      while (questions.length > count) do
        questions.delete_at(rand(questions.length))
      end
      questions.shuffle! if mix_questions
      if mix_choices
        questions.each do |question|
          question.choices.shuffle!
        end
      end
      @categories[category_id] = {:category => category, :count => count, :questions => questions} if count > 0
    end
    if just_print
      render(:layout => "print")
    else
      ok = true
      exam = Exam.new(params[:exam])
      exam.examiner = current_user
      exam.enlapsed_time = 0
      if !(exam.save)
        ok = false
      else
        position = 0
        @categories.values.each do |category|
          category[:questions].each do |question|
            answer = Answer.new
            answer.exam = exam
            answer.question = question
            answer.position = position
            answer.enlapsed_time = 0
            answer.choices_order = question.choices.collect{|choice| choice.id}.join(",")
            answer.save
            position += 1
          end
        end
      end
      if !ok
        redirect_to(:action => :new, :notice => t(:error_saving_object))
      else
        redirect_to(:controller => :user_sessions,
                    :action => :change_user,
                    :user_session => {:login => Setting.exams_login, :password => Setting.exams_password},
                    :back_url => url_for(:action => :question_new, :id => exam.id, :position => -1))
      end
    end
  end

  def edit
  end

  def update
    if @exam.update_attributes(params[:exam])
      redirect_to(:action => :show, :id => @exam.id, :notice => t(:notice_exam_updated))
    else
      render(:action => :edit, :id => @exam.id)
    end
  end

  def destroy
    @exam.destroy
    redirect_to(exams_url)
  end

  def question_new
    if (params[:position].nil? or (params[:position].to_i < 0) or @answer.nil?)
      # Render instructions.
      render(:instructions, :layout => "exam")
    else
      @answer.rendered_at = Time.new
      @answer.save
      render(:layout => "exam")
    end
  end

  def question_create
    @answer.choice_id = params[:choice_id]
    if params[:reset]
      @answer.choice_id = nil
    end
    if @answer.rendered_at
      time = Time.new - @answer.rendered_at
      @answer.enlapsed_time += time
      @exam.enlapsed_time += time
    end
    if !(@answer.save) or !(@exam.save)
      redirect_to(:action => :question_new, :id => @exam.id, :position => @answer.position, :notice => t(:error_saving_object))
    elsif params[:end]
      redirect_to(:action => :end, :id => @exam.id)
    elsif params[:reset]
      redirect_to(:action => :question_new, :id => @exam.id, :position => @answer.position)
    elsif params[:first]
      redirect_to(:action => :question_new, :id => @exam.id, :position => 0)
    elsif params[:last]
      redirect_to(:action => :question_new, :id => @exam.id, :position => @exam.questions.length - 1)
    else
      redirect_to(:action => :question_new, :id => @exam.id, :position => @answer.position + (params[:next] ? 1 : -1))
    end
  end

  def end
    render(:layout => "exam")
  end

private

  def find_exam
    @exam = Exam.find(params[:id])
  rescue
    @exam = nil
  end

  def find_answer
    @answer = Answer.find_by_exam_id_and_position(params[:id], params[:position])
  rescue
    @answer = nil
  end

end
