class QuestionsController < ApplicationController

  before_filter :find_question, :only => [:show, :edit, :update, :destroy, :enable, :disable]
  before_filter :find_category_id, :only => [:index, :new, :create]
  before_filter :find_page

  def allows_to?(action)
    allows_to = case(action)
    when "index", "show", "new", "create", "edit", "update", "enable", "disable" then current_user.has_role?("editor")
    when "destroy" then current_user.has_role?("admin")
    else false
    end
    return(allows_to)
  end

  def index
    @questions = Question.where(:category_id => @category_id).paginate(:page => @page, :per_page => Setting.elements_per_page)
  end

  def show
  end

  def new
    @question = Question.new
    @question.category_id = @category_id
    (0..3).each do |i|
      choice = Choice.new
      choice.position = i
      @question.choices << choice
    end
    10.times {@question.assets.build}
  end

  def create
    @question = Question.new(params[:question])
    params[:choices].each_pair do |position, choice_params|
      choice = Choice.new(choice_params)
      choice.position = position.to_i
      @question.choices << choice
    end
    @question.author = current_user
    if @question.save
      if params[:commit]
        redirect_to({:action => :show, :id => @question.id, :category_id => @category_id, :page => @page}, :notice => t(:notice_question_created))
      else
        redirect_to({:action => :new, :category_id => @category_id, :page => @page}, :notice => t(:notice_question_created))
      end
    else
      render(:action => :new)
    end
  end

  def edit
    10.times {@question.assets.build}
  end

  def update
    previous_question = @question.dup
    previous_question.choices = @question.choices.dup
    error = false
    params[:choices].each_pair do |position, choice_params|
      choice = @question.choices.find_by_position(position)
      choice_params[:is_correct] = 0 unless defined?(choice_params[:is_correct]) and choice_params[:is_correct] == "1"
      if (!(choice.update_attributes(choice_params)))
        error = true
      end
    end
    @question = Question.find(params[:id])
    @question.author = current_user unless @question.author
    if ((!error) and @question.update_attributes(params[:question]))
      @question.create_journal(previous_question)
      redirect_to({:action => :show, :id => @question.id, :category_id => @category_id, :page => @page}, :notice => t(:notice_question_updated))
    else
      render(:action => :edit)
    end
  end

  def destroy
    @question.destroy
    redirect_to({:action => :index, :category_id => @category_id, :page => @page})
  end

  def enable
    previous_question = @question.dup
    @question.active = true
    if (@question.save)
      @question.create_journal(previous_question)
      notice = t(:notice_question_updated)
    else
      notice = t(:error_saving_object)
    end
    redirect_to(params[:back_url] ? params[:back_url] : root_url, :notice => notice)
  end

  def disable
    previous_question = @question.dup
    @question.active = false
    if (@question.save)
      @question.create_journal(previous_question)
      notice = t(:notice_question_updated)
    else
      notice = t(:error_saving_object)
    end
    redirect_to(params[:back_url] ? params[:back_url] : root_url, :notice => notice)
  end

private
  
  def find_question
    if (!(params[:id].nil?))
      @question = Question.find(params[:id])
      @category_id = @question.category_id unless @question.nil?
    end
  end
  
  def find_category_id
    if (!(params[:category_id].nil?))
      @category_id = params[:category_id].to_i
    end
  end

  def find_page
    @page = params[:page].nil? ? 1 : params[:page].to_i
  end

end
