class Question < ActiveRecord::Base

  belongs_to :category
  has_many :choices, :order => :position, :dependent => :destroy
  belongs_to :author, :class_name => "User"
  has_many :assets, :as => :attachable, :dependent => :destroy
  accepts_nested_attributes_for :assets, :allow_destroy => true
  has_many :answers, :dependent => :destroy
  has_many :exams, :through => :answers
  has_many :journals, :as => :journalized, :dependent => :destroy
  validates_presence_of :text
  validate :validate_one_choice

  def validate_one_choice
    correct_choices = 0
    choices.each do |choice|
      correct_choices += 1 if choice.is_correct
    end
    errors.add(:base, :one_choice) if correct_choices != 1
  end

  def correct_choice
    the_correct_choice = nil
    choices.each do |choice|
      if the_correct_choice.nil? and choice.is_correct?
        the_correct_choice = choice
      end
    end
    return(the_correct_choice)
  end

  def statistics
    if @statistics.nil?
      @statistics = {:total => 0, :ok => 0, :ko => 0, :empty => 0}
      answers.each do |answer|
        @statistics[:total] += 1
        if answer.choice.nil?
          @statistics[:empty] += 1
        elsif answer.is_correct?
          @statistics[:ok] += 1
        else
          @statistics[:ko] += 1
        end
      end
    end
    return(@statistics)
  end

  def create_journal(previous_question)
    details = changes_since(previous_question)
    if (details.any?)
      journal = Journal.new(:journalized => self, :user => author, :details => details)
      journal.save!
    end
  end

private

  def changes_since(previous)
    details = []
    (Question.column_names - ["id", "created_at", "updated_at"]).each do |column|
      old_value = previous.send(column)
      new_value = send(column)
      if ((old_value != new_value) and ((!(old_value.blank?)) or (!(new_value.blank?))))
        details << JournalDetail.new(:key => column, :old_value => old_value, :new_value => new_value)
      end
    end
    choices.each do |choice|
      previous_choice_index = previous.choices.index{|previous_choice| previous_choice.id == choice.id}
      if (!(previous_choice_index.nil?))
        details += choice.changes_since(previous.choices[previous_choice_index], "choice_#{previous_choice_index}_")
      end
    end
    return(details)
  end

end
